import dbnd

from targets.marshalling import register_marshaller
from targets.target_config import FileFormat
from targets.values import register_value_type


@dbnd.hookimpl
def dbnd_setup_plugin():
    import tensorflow

    from dbnd_tensorflow.marshalling.tensorflow_marshaller import (
        TensorflowKerasHistoryMarshaller,
        TensorflowKerasModelMarshaller,
    )
    from dbnd_tensorflow.marshalling.tensorflow_values import (
        TensorflowHistoryValueType,
        TensorflowModelValueType,
    )

    register_marshaller(
        tensorflow.python.keras.engine.training.Model,
        FileFormat.tfmodel,
        TensorflowKerasModelMarshaller(),
    )
    register_marshaller(
        tensorflow.python.keras.callbacks.History,
        FileFormat.tfhistory,
        TensorflowKerasHistoryMarshaller(),
    )

    register_value_type(TensorflowModelValueType())
    register_value_type(TensorflowHistoryValueType())



import tensorflow

from dbnd._core.errors import friendly_error
from dbnd._vendor.cloudpickle import cloudpickle
from targets import LocalFileSystem
from targets.marshalling.marshaller import Marshaller


class TensorflowKerasModelMarshaller(Marshaller):
    type = tensorflow.keras.models.Model
    support_directory_read = False

    _compression_read_arg = None
    _compression_write_arg = None

    support_cache = False
    disable_default_index = False

    def target_to_value(self, target, **kwargs):
        if not isinstance(target.fs, LocalFileSystem):
            raise friendly_error.targets.target_must_be_local_for_tensorflow_marshalling(
                target
            )
        model = tensorflow.keras.models.load_model(target.path, **kwargs)
        return model

    def value_to_target(self, value, target, **kwargs):
        tensorflow.keras.models.save_model(value, target.path, **kwargs)


class TensorflowKerasHistoryMarshaller(Marshaller):
    type = tensorflow.python.keras.callbacks.History

    def target_to_value(self, target, **kwargs):
        h = tensorflow.python.keras.callbacks.History()
        with target.open("rb") as dumped:
            h.history = cloudpickle.load(dumped)
        return h

    def value_to_target(self, value, target, **kwargs):
        with target.open("wb") as dumped:
            cloudpickle.dump(obj=value.history, file=dumped)



from os import path

import setuptools

from setuptools.config import read_configuration


BASE_PATH = path.dirname(__file__)
CFG_PATH = path.join(BASE_PATH, "setup.cfg")

config = read_configuration(CFG_PATH)
version = config["metadata"]["version"]

setuptools.setup(
    name="dbnd-tensorflow",
    package_dir={"": "src"},
    install_requires=["dbnd==" + version, "tensorflow", "h5py==2.10.0"],
    extras_require=dict(tests=[]),
    entry_points={"dbnd": ["dbnd-tensorflow = dbnd_tensorflow._plugin"]},
)
